##############################################################################
#
# Copyright (c) 2002 Nexedi SARL and Contributors. All Rights Reserved.
#                    Sebastien Robin <seb@nexedi.com>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

from Products.ERP5.Document.Organisation import Organisation

class ERP5Conduit:
    """
        A conduit is a piece of code in charge of
        
        - updating an object attributes from an XUpdate XML stream
        
        (Conduits are not in charge of creating new objects which
        are eventually missing in a synchronisation process)

        If an object has be created during a synchronisation process,
        the way to proceed consists in:
        
        1- creating an empty instance of the appropriate class
           in the appropriate directory
        
        2- updating that empty instance with the conduit
        
        The first implementation of ERP5 synchronisation 
        will define a default location to create new objects and
        a default class. This will be defined at the level of the synchronisation
        tool
    """

    def newObject(self, object, action):
        """
            modify the object with datas from
            the xml (action section)
        """
        for subnode in action.childNodes:
            if subnode.nodeType == subnode.ELEMENT_NODE and subnode.nodeName == 'Item':
                for subnode2 in subnode.childNodes:
                    if subnode2.nodeType == subnode2.ELEMENT_NODE and subnode2.nodeName == 'Data':
                        for subnode3 in subnode2.childNodes:
                            if subnode3.nodeType == subnode3.ELEMENT_NODE and subnode3.nodeName != 'id':
                                # edit the object with a dictionnary of arguments, like {"telephone_number":"02-5648"}
                                keyword=str(subnode3.nodeName)
                                args= {keyword:str(subnode3.childNodes[0].data)}
                                object.edit(**args)

    def ApplyUpdate(self, object, xupdate_stream):
        """
            update an abject with an xupdate file
        """
        for subnode in xupdate_stream.childNodes:
            if subnode.nodeType == subnode.ELEMENT_NODE and subnode.nodeName == 'xupdate:modifications':
                for subnode2 in subnode.childNodes:
                    if subnode2.nodeType == subnode2.ELEMENT_NODE:
                        for subnode3 in subnode2.attributes:
                            if subnode3.nodeType == subnode3.ATTRIBUTE_NODE:
                                # ex: get telephone_number in "/tmp[1]/telephone_number[1]"
                                keyword = str(subnode3.nodeValue.split('/')[2].split('[')[0])

                        data = None
                        # This is the case where the local value is None, so there's
                        # a subnode <xupdate:text>
                        for subnode3 in subnode2.childNodes:
                            for subnode4 in subnode3.childNodes:
                                data = str(subnode4.data.split('\n')[1])

                        # This is the case where the local value is not None, so there's not
                        # a subnode <xupdate:text>
                        #data = "toto"
                        if data is None:
                            for subnode3 in subnode2.childNodes:
                                data = str(subnode3.data.split('\n')[1])

                        args= {keyword:data}
                        if keyword != "id":
                            object.edit(**args)

