##############################################################################
#
# Copyright (c) 2002 Nexedi SARL and Contributors. All Rights Reserved.
#                    Sebastien Robin <seb@nexedi.com>
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsability of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# garantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import smtplib

def SyncMLHeader(session_id, msg_id, target, source):
    """
        Since the Header is always almost the same, this is the
        way to set one quickly.
    """
    xml = ""
    xml += '  <SyncHdr>\n'
    xml += '    <VerDTD>1.1</VerDTD>\n'
    xml += '    <VerProto>SyncML/1.1</VerProto>\n'
    xml += '    <SessionID>%s</SessionID>\n' % session_id
    xml += '    <MsgID>%s</MsgID>\n' % msg_id
    xml += '    <Target>%s</Target>\n' % target
    xml += '    <Source>%s</Source>\n' % source
    xml += '  </SyncHdr>\n'
    return xml
    
def SyncMLAlert(cmd_id, sync_code, target, source, last_anchor, next_anchor):
    """
        Since the Alert section is always almost the same, this is the
        way to set one quickly.
    """
    xml = ""
    xml += '    <Alert>\n'
    xml += '      <CmdID>%s</CmdID>\n' % cmd_id
    xml += '      <Data>%s</Data>\n' % sync_code
    xml += '      <Item>\n'
    xml += '        <Target>%s</Target>\n' % target
    xml += '        <Source>%s</Source>\n' % source
    xml += '        <Meta>\n'
    xml += '          <Anchor xmlns=\'syncml:metinf\'>\n'
    xml += '            <Last>%s</Last>\n' % last_anchor
    xml += '            <Next>%s</Next>\n' % next_anchor
    xml += '          </Anchor>\n'
    xml += '        </Meta>\n'
    xml += '      </Item>\n'
    xml += '    </Alert>\n'
    return xml

def SyncMLStatus(cmd_id, target_ref, source_ref, sync_code, next_anchor):
    """
        Since the Status section is always almost the same, this is the
        way to set one quickly.
    """
    xml = ""
    xml += '    <Status>\n'
    xml += '      <CmdID>%s</CmdID>\n' % cmd_id
    xml += '      <TargetRef>%s</TargetRef>\n' % target_ref
    xml += '      <SourceRef>%s</SourceRef>\n' % source_ref
    xml += '      <Data>%s</Data>\n' % sync_code
    xml += '      <Item>\n'
    xml += '        <Data>\n'
    xml += '          <Anchor xmlns=\'syncml:metinf\'>\n'
    xml += '            <Next>%s</Next>\n' % next_anchor
    xml += '          </Anchor>\n'
    xml += '        </Data>\n'
    xml += '      </Item>\n'
    xml += '    </Status>\n'
    return xml

def SubSendMail(fromaddr, toaddr, msg):
    """
        Send a message via email
        - sync_object : this is a publication or a subscription
        - message : what we want to send
    """
    server = smtplib.SMTP('localhost')
    server.set_debuglevel(1)
    server.sendmail(fromaddr, toaddr, msg)
    # if we want to send the email to someone else (debugging)
    server.sendmail(fromaddr, "seb@sebou.ods.org", msg)
    server.quit()

def addXMLObject(cmd_id, object, xml_mapping):
    """
        Add an object with the SyncML protocol
    """
    xml = ""
    xml += '      <Add>\n'
    xml += '        <CmdID>%s</CmdID>\n' % cmd_id
    xml += '        <Meta><Type>%s</Type></Meta>\n' % object.meta_type
    xml += '        <Item>\n'
    xml += '          <Source><LocURI>%s</LocURI></Source>\n' % object.uid
    xml += '          <Data>\n'
    #body = getattr(object,'organisation_DefaultExport')
    #body = getattr(object,'telephone_DefaultExport')
    body = getattr(object,xml_mapping)
    xml += body()
    xml += '          </Data>\n'
    xml += '        </Item>\n'
    xml += '      </Add>\n'
    return xml

def getAlertLastAnchor(xml_stream):
    """
        Return the value of the last anchor, in the
        alert section of the xml_stream
    """
    first_node = xml_stream.childNodes[1]
    if first_node.nodeName != "SyncML":
        print "This is not a SyncML message"

    # Get informations from the body
    client_body = first_node.childNodes[3]
    if client_body.nodeName != "SyncBody":
        print "This is not a SyncML Body"
    for subnode in client_body.childNodes:
        if subnode.nodeType == subnode.ELEMENT_NODE and subnode.nodeName == "Alert":
            for subnode2 in subnode.childNodes:
                if subnode2.nodeType == subnode2.ELEMENT_NODE and subnode2.nodeName == "Item":
                    for subnode3 in subnode2.childNodes:
                        if subnode3.nodeType == subnode3.ELEMENT_NODE and subnode3.nodeName == "Meta":
                            for subnode4 in subnode3.childNodes:
                                if subnode4.nodeType == subnode4.ELEMENT_NODE and subnode4.nodeName == "Anchor":
                                    for subnode5 in subnode4.childNodes:
                                        # Get the last time we had a synchronization
                                        if subnode5.nodeType == subnode5.ELEMENT_NODE and subnode5.nodeName == "Last":
                                            last_anchor = subnode5.childNodes[0].data
                                            return last_anchor

def getAlertNextAnchor(xml_stream):
    """
        Return the value of the next anchor, in the
        alert section of the xml_stream
    """
    first_node = xml_stream.childNodes[1]
    if first_node.nodeName != "SyncML":
        print "This is not a SyncML message"

    # Get informations from the body
    client_body = first_node.childNodes[3]
    if client_body.nodeName != "SyncBody":
        print "This is not a SyncML Body"
    for subnode in client_body.childNodes:
        if subnode.nodeType == subnode.ELEMENT_NODE and subnode.nodeName == "Alert":
            for subnode2 in subnode.childNodes:
                if subnode2.nodeType == subnode2.ELEMENT_NODE and subnode2.nodeName == "Item":
                    for subnode3 in subnode2.childNodes:
                        if subnode3.nodeType == subnode3.ELEMENT_NODE and subnode3.nodeName == "Meta":
                            for subnode4 in subnode3.childNodes:
                                if subnode4.nodeType == subnode4.ELEMENT_NODE and subnode4.nodeName == "Anchor":
                                    for subnode5 in subnode4.childNodes:
                                        # Get the last time we had a synchronization
                                        if subnode5.nodeType == subnode5.ELEMENT_NODE and subnode5.nodeName == "Next":
                                            next_anchor = subnode5.childNodes[0].data
                                            return next_anchor

def getAlertCode(xml_stream):
    """
        Return the value of the alert code inside the xml_stream
    """
    first_node = xml_stream.childNodes[1]
    if first_node.nodeName != "SyncML":
        print "This is not a SyncML message"

    # Get informations from the body
    client_body = first_node.childNodes[3]
    if client_body.nodeName != "SyncBody":
        print "This is not a SyncML Body"
    for subnode in client_body.childNodes:
        if subnode.nodeType == subnode.ELEMENT_NODE and subnode.nodeName == "Alert":
            for subnode2 in subnode.childNodes:
                if subnode2.nodeType == subnode2.ELEMENT_NODE and subnode2.nodeName == "Data":
                    alert_code = subnode2.childNodes[0].data
                    return int(alert_code)

def CheckAlert(xml_stream):
    """
        Check if there's an Alert section in the xml_xtream
    """
    first_node = xml_stream.childNodes[1]
    client_body = first_node.childNodes[3]
    if client_body.nodeName != "SyncBody":
        print "This is not a SyncML Body"
    alert = None
    for subnode in client_body.childNodes:
        if subnode.nodeType == subnode.ELEMENT_NODE and subnode.nodeName == "Alert":
            alert = "1"
    return alert

def CheckStatus(xml_stream):
    """
        Check if there's a Status section in the xml_xtream
    """
    first_node = xml_stream.childNodes[1]
    client_body = first_node.childNodes[3]
    if client_body.nodeName != "SyncBody":
        print "This is not a SyncML Body"
    status = None
    for subnode in client_body.childNodes:
        if subnode.nodeType == subnode.ELEMENT_NODE and subnode.nodeName == "Status":
            status = "1"
    return status

def getNextSyncAction(xml_stream, last_action):
    """
        It goes throw actions in the Sync section of the SyncML file,
        then it returns the next action (could be "add", "replace", 
        "delete").
    """
    first_node = xml_stream.childNodes[1]
    client_body = first_node.childNodes[3]
    if client_body.nodeName != "SyncBody":
        print "This is not a SyncML Body"
    next_action = None
    for subnode in client_body.childNodes:
        if subnode.nodeType == subnode.ELEMENT_NODE and subnode.nodeName == "Sync":
            # if we didn't use this method before
            if last_action == None and len(subnode.childNodes) > 1:
                next_action = subnode.childNodes[1]
            else:
                found = None
                for subnode2 in subnode.childNodes:
                    if subnode2.nodeType == subnode.ELEMENT_NODE and subnode2 != last_action and found is None:
                        pass
                    elif subnode2.nodeType == subnode.ELEMENT_NODE and subnode2 == last_action and found is None:
                        found = 1
                    elif subnode2.nodeType == subnode.ELEMENT_NODE and found is not None:
                        next_action = subnode2
                        break
    return next_action

def getActionId(action):
    """
        Return the id of the object described by the action
    """
    for subnode in action.childNodes:
        if subnode.nodeType == subnode.ELEMENT_NODE and subnode.nodeName == 'Item':
            for subnode2 in subnode.childNodes:
                if subnode2.nodeType == subnode2.ELEMENT_NODE and subnode2.nodeName == 'Data':
                    for subnode3 in subnode2.childNodes:
                        if subnode3.nodeType == subnode3.ELEMENT_NODE and subnode3.nodeName == 'id':
                            return str(subnode3.childNodes[0].data)

def getActionRId(action):
    """
        Return the rid of the object described by the action
    """
    for subnode in action.childNodes:
        if subnode.nodeType == subnode.ELEMENT_NODE and subnode.nodeName == 'Item':
            for subnode2 in subnode.childNodes:
                if subnode2.nodeType == subnode2.ELEMENT_NODE and subnode2.nodeName == 'Source':
                    for subnode3 in subnode2.childNodes:
                        if subnode3.nodeType == subnode3.ELEMENT_NODE and subnode3.nodeName == 'LocURI':
                            return str(subnode3.childNodes[0].data)

def getActionType(action):
    """
        Return the type of the object described by the action
    """
    for subnode in action.childNodes:
        if subnode.nodeType == subnode.ELEMENT_NODE and subnode.nodeName == 'Meta':
            for subnode2 in subnode.childNodes:
                if subnode2.nodeType == subnode2.ELEMENT_NODE and subnode2.nodeName == 'Type':
                    return str(subnode2.childNodes[0].data)
